@@ -4,18 +4,16 @@ dicom2nifti
 
 @author: abrys
 """
-import dicom2nifti.compressed_dicom as compressed_dicom
-
+import logging
 import os
 import struct
 
+import numpy
+import pydicom
 from pydicom.tag import Tag
 
-import logging
-import numpy
-
-from dicom2nifti.exceptions import ConversionValidationError, ConversionError
 import dicom2nifti.settings
+from dicom2nifti.exceptions import ConversionValidationError, ConversionError
 
 logger = logging.getLogger(__name__)
 
@@ -36,11 +34,11 @@ def read_dicom_directory(dicom_directory, stop_before_pixels=False):
     for root, _, files in os.walk(dicom_directory):
         for dicom_file in files:
             file_path = os.path.join(root, dicom_file)
-            if compressed_dicom.is_dicom_file(file_path):
-                dicom_headers = compressed_dicom.read_file(file_path,
-                                                           defer_size="1 KB",
-                                                           stop_before_pixels=stop_before_pixels,
-                                                           force=dicom2nifti.settings.pydicom_read_force)
+            if is_dicom_file(file_path):
+                dicom_headers = pydicom.read_file(file_path,
+                                                  defer_size="1 KB",
+                                                  stop_before_pixels=stop_before_pixels,
+                                                  force=dicom2nifti.settings.pydicom_read_force)
                 if is_valid_imaging_dicom(dicom_headers):
                     dicom_input.append(dicom_headers)
     return dicom_input
@@ -523,6 +521,7 @@ def write_bval_file(bvals, bval_file):
         # join the bvals using a space and write to the file
         text_file.write('%s\n' % ' '.join(map(str, bvals)))
 
+
 def multiframe_create_affine(dicoms):
     """
     Function to generate the affine matrix for a dicom series
@@ -533,12 +532,19 @@ def multiframe_create_affine(dicoms):
 
     # Create affine matrix (http://nipy.sourceforge.net/nibabel/dicom/dicom_orientation.html#dicom-slice-affine)
     frame_info = dicoms[0].PerFrameFunctionalGroupsSequence
-    image_orient1 = numpy.array(frame_info[0].PlaneOrientationSequence[0].ImageOrientationPatient)[0:3]
-    image_orient2 = numpy.array(frame_info[0].PlaneOrientationSequence[0].ImageOrientationPatient)[3:6]
-    first_image_pos = numpy.array(frame_info[0].PlanePositionSequence[0].ImagePositionPatient)
+    image_orient1, image_orient2 = _multiframe_get_image_orientations(dicoms[0], 0)
 
-    delta_r = float(frame_info[0].PixelMeasuresSequence[0].PixelSpacing[0])
-    delta_c = float(frame_info[0].PixelMeasuresSequence[0].PixelSpacing[1])
+    if 'PixelMeasuresSequence' in frame_info[0]:
+        delta_r = float(frame_info[0].PixelMeasuresSequence[0].PixelSpacing[0])
+        delta_c = float(frame_info[0].PixelMeasuresSequence[0].PixelSpacing[1])
+    elif "SharedFunctionalGroupsSequence" in dicoms[0] and \
+            "PixelMeasuresSequence" in dicoms[0].SharedFunctionalGroupsSequence[0]:
+        shared_frame_info = dicoms[0].SharedFunctionalGroupsSequence
+        delta_r = numpy.array(shared_frame_info[0].PixelMeasuresSequence[0].PixelSpacing[0])
+        delta_c = numpy.array(shared_frame_info[0].PixelMeasuresSequence[0].PixelSpacing[1])
+    else:
+        logger.warning('Unsupported or missing PixelMeasuresSequence')
+        raise ConversionError('Unsupported or missing PixelMeasuresSequence')
 
     image_pos = numpy.array(frame_info[0].PlanePositionSequence[0].ImagePositionPatient)
 
@@ -565,6 +571,7 @@ def multiframe_create_affine(dicoms):
     )
     return affine, numpy.linalg.norm(step)
 
+
 def create_affine(sorted_dicoms):
     """
     Function to generate the affine matrix for a dicom series
@@ -605,6 +612,7 @@ def create_affine(sorted_dicoms):
     )
     return affine, numpy.linalg.norm(step)
 
+
 def multiframe_validate_orthogonal(dicoms):
     """
     Validate that volume is orthonormal
@@ -615,6 +623,7 @@ def multiframe_validate_orthogonal(dicoms):
     if not multiframe_is_orthogonal(dicoms, log_details=True):
         raise ConversionValidationError('NON_CUBICAL_IMAGE/GANTRY_TILT')
 
+
 def validate_orthogonal(dicoms):
     """
     Validate that volume is orthonormal
@@ -627,6 +636,7 @@ def validate_orthogonal(dicoms):
     if not is_orthogonal(dicoms, log_details=True):
         raise ConversionValidationError('NON_CUBICAL_IMAGE/GANTRY_TILT')
 
+
 def multiframe_is_orthogonal(dicoms, log_details=False):
     """
     Validate that volume is orthonormal
@@ -634,8 +644,8 @@ def multiframe_is_orthogonal(dicoms, log_details=False):
     :param dicoms: check that we have a volume without skewing
     """
     frame_info = dicoms[0].PerFrameFunctionalGroupsSequence
-    first_image_orient1 = numpy.array(frame_info[0].PlaneOrientationSequence[0].ImageOrientationPatient)[0:3]
-    first_image_orient2 = numpy.array(frame_info[0].PlaneOrientationSequence[0].ImageOrientationPatient)[3:6]
+
+    first_image_orient1, first_image_orient2 = _multiframe_get_image_orientations(dicoms[0], 0)
     first_image_pos = numpy.array(frame_info[0].PlanePositionSequence[0].ImagePositionPatient)
 
     last_image_pos = numpy.array(frame_info[-1].PlanePositionSequence[0].ImagePositionPatient)
@@ -657,6 +667,7 @@ def multiframe_is_orthogonal(dicoms, log_details=False):
         return False
     return True
 
+
 def is_orthogonal(dicoms, log_details=False):
     """
     Validate that volume is orthonormal
@@ -735,6 +746,7 @@ def sort_dicoms(dicoms):
     if diff_z >= diff_x and diff_z >= diff_y:
         return dicom_input_sorted_z
 
+
 def multiframe_validate_slice_increment(dicoms):
     """
     Validate that the distance between all slices is equal (or very close to)
@@ -759,6 +771,7 @@ def multiframe_validate_slice_increment(dicoms):
             raise ConversionValidationError('SLICE_INCREMENT_INCONSISTENT')
         previous_image_position = current_image_position
 
+
 def validate_slice_increment(dicoms):
     """
     Validate that the distance between all slices is equal (or very close to)
@@ -867,11 +880,11 @@ def multiframe_validate_slicecount(dicoms):
     """
     frame_info = dicoms[0].PerFrameFunctionalGroupsSequence
     if len(frame_info) <= 3:
-
         logger.warning('At least 3 slices are needed for correct conversion')
         logger.warning('---------------------------------------------------------')
         raise ConversionValidationError('TOO_FEW_SLICES/LOCALIZER')
 
+
 def validate_slicecount(dicoms):
     """
     Validate that volume is big enough to create a meaningfull volume
@@ -884,19 +897,37 @@ def validate_slicecount(dicoms):
         logger.warning('---------------------------------------------------------')
         raise ConversionValidationError('TOO_FEW_SLICES/LOCALIZER')
 
+
+def _multiframe_get_image_orientations(dicom_headers, frame_number=0):
+    frame_info = dicom_headers.PerFrameFunctionalGroupsSequence
+    if "PlaneOrientationSequence" in frame_info[frame_number]:
+        image_orient1 = numpy.array(frame_info[frame_number].PlaneOrientationSequence[0].ImageOrientationPatient)[0:3]
+        image_orient2 = numpy.array(frame_info[frame_number].PlaneOrientationSequence[0].ImageOrientationPatient)[3:6]
+    elif "SharedFunctionalGroupsSequence" in dicom_headers and \
+            "PlaneOrientationSequence" in dicom_headers.SharedFunctionalGroupsSequence[0]:
+        shared_frame_info = dicom_headers.SharedFunctionalGroupsSequence
+        image_orient1 = numpy.array(shared_frame_info[0].PlaneOrientationSequence[0].ImageOrientationPatient)[0:3]
+        image_orient2 = numpy.array(shared_frame_info[0].PlaneOrientationSequence[0].ImageOrientationPatient)[3:6]
+    else:
+        logger.warning('Unsupported or missing PlaneOrientationSequence')
+        raise ConversionError('Unsupported or missing PlaneOrientationSequence')
+
+    return image_orient1, image_orient2
+
+
 def multiframe_validate_orientation(dicoms):
     """
     Validate that all dicoms have the same orientation
 
     :param dicoms: list of dicoms
     """
+
     frame_info = dicoms[0].PerFrameFunctionalGroupsSequence
-    first_image_orient1 = numpy.array(frame_info[0].PlaneOrientationSequence[0].ImageOrientationPatient)[0:3]
-    first_image_orient2 = numpy.array(frame_info[0].PlaneOrientationSequence[0].ImageOrientationPatient)[3:6]
-    for frame_ in frame_info:
+    first_image_orient1, first_image_orient2 = _multiframe_get_image_orientations(dicoms[0], 0)
+
+    for frame_index, frame_ in enumerate(frame_info):
         # Create affine matrix (http://nipy.sourceforge.net/nibabel/dicom/dicom_orientation.html#dicom-slice-affine)
-        image_orient1 = numpy.array(frame_.PlaneOrientationSequence[0].ImageOrientationPatient)[0:3]
-        image_orient2 = numpy.array(frame_.PlaneOrientationSequence[0].ImageOrientationPatient)[3:6]
+        image_orient1, image_orient2 = _multiframe_get_image_orientations(dicoms[0], frame_index)
         if not numpy.allclose(image_orient1, first_image_orient1, rtol=0.001, atol=0.001) \
                 or not numpy.allclose(image_orient2, first_image_orient2, rtol=0.001, atol=0.001):
             logger.warning('Image orientations not consistent through all slices')
@@ -906,6 +937,7 @@ def multiframe_validate_orientation(dicoms):
             logger.warning('---------------------------------------------------------')
             raise ConversionValidationError('IMAGE_ORIENTATION_INCONSISTENT')
 
+
 def validate_orientation(dicoms):
     """
     Validate that all dicoms have the same orientation
@@ -950,3 +982,29 @@ def get_nifti_data(nifti_image):
     Function that replicates the deprecated nifti.get_data behavior
     """
     return numpy.asanyarray(nifti_image.dataobj)
+
+
+def is_dicom_file(filename):
+    """
+    Util function to check if file is a dicom file
+    the first 128 bytes are preamble
+    the next 4 bytes should contain DICM otherwise it is not a dicom
+
+    :param filename: file to check for the DICM header block
+    :type filename: str
+    :returns: True if it is a dicom file
+    """
+    file_stream = open(filename, 'rb')
+    file_stream.seek(128)
+    data = file_stream.read(4)
+    file_stream.close()
+    if data == b'DICM':
+        return True
+    if dicom2nifti.settings.pydicom_read_force:
+        try:
+            dicom_headers = pydicom.read_file(filename, defer_size="1 KB", stop_before_pixels=True, force=True)
+            if dicom_headers is not None:
+                return True
+        except:
+            pass
+    return False
